/*
 * todolist.c
 */

/*
 * yank  -  yet another NoteKeeper
 * Copyright (C) 1999 Michael Humann <m.hussmann@home.ins.de>
 *
 * This program is free software; you  can redistribute it and/or modify it
 * under the terms of  the GNU General  Public License as published  by the
 * Free Software Foundation;  either version 2 of  the License, or (at your
 * option) any later version.
 *
 * This program  is  distributed in the  hope  that it will  be useful, but
 * WITHOUT  ANY   WARRANTY;  without   even   the  implied    warranty   of
 * MERCHANTABILITY or    FITNESS FOR A PARTICULAR   PURPOSE.   See  the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the  GNU General Public License along
 * with this program; if not, write to the  Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "todolist.h"

/* listwidget */
extern GtkWidget *todolist;

/* ------------------------------------------------------ */
/*
 * add an entry to the list
 */

void
add_todo_item(note_data *item)
{
    static gchar *txt[] = { NULL, NULL, NULL, NULL };
    gint   row;

    if (item)
    {
        txt[0] = item->date;
        txt[1] = item->prio;
        txt[2] = item->title;
        row = gtk_clist_append(GTK_CLIST(todolist), txt);
        gtk_clist_set_row_data(GTK_CLIST(todolist), row, item);
    }
}

/* ------------------------------------------------------ */
/*
 * modify/ remove an entry
 */

void
edit_todo_item(note_data *old_i, note_data *new_i)
{
    gint pos;

    if (new_i->todo == 1)
    {
        delete_todo_item(old_i);
    }
    else
    {
        pos = gtk_clist_find_row_from_data(GTK_CLIST(todolist), old_i);
        if (pos != -1)
        {
            gtk_clist_set_text(GTK_CLIST(todolist), pos, 0, new_i->date);
            gtk_clist_set_text(GTK_CLIST(todolist), pos, 1, new_i->prio);
            gtk_clist_set_text(GTK_CLIST(todolist), pos, 2, new_i->title);
            gtk_clist_set_row_data(GTK_CLIST(todolist), pos, new_i);
        }
        else
        {
            add_todo_item(new_i);
        }
    }
}

/* ------------------------------------------------------ */
/*
 * remove an entry from the list
 */

void
delete_todo_item(note_data *item)
{
    gint pos;

    pos = gtk_clist_find_row_from_data(GTK_CLIST(todolist), item);
    if (pos != -1)
    {
        gtk_clist_remove(GTK_CLIST(todolist), pos);
    }
    else
    {
        gnome_warning_dialog(_("Todolist (delete): Can't find note!"));
    }
}

/* ------------------------------------------------------ */
/*
 * sort notes by date
 */

gint
sort_todo_items_date(GtkCList *list, gconstpointer ptr1,
                          gconstpointer ptr2)
{
    GtkCListRow *row1;
    GtkCListRow *row2;
    glong       days1;
    glong       days2;
    glong       cdate;

    row1 = (GtkCListRow *)ptr1;
    row2 = (GtkCListRow *)ptr2;

    cdate = date2days(current_date());
    days1 = cdate - get_note_days((note_data *)row1->data);
    days2 = cdate - get_note_days((note_data *)row2->data);
    
    return (days2 - days1);
}

/* ------------------------------------------------------ */
/*
 * sort notes by priority
 */

gint
sort_todo_items_prio(GtkCList *list, gconstpointer ptr1,
                     gconstpointer ptr2)
{
    GtkCListRow *row1;
    GtkCListRow *row2;
    gint        p1;
    gint        p2;
    
    row1 = (GtkCListRow *)ptr1;
    row2 = (GtkCListRow *)ptr2;

    p1 = get_note_prio(((note_data *)row1->data));
    p2 = get_note_prio(((note_data *)row2->data));
    
    return (p1 - p2);
}

/* ------------------------------------------------------ */
/*
 * sort notes by date& priority
 */

gint
sort_todo_items_date_prio(GtkCList *w, gconstpointer ptr1, gconstpointer ptr2)
{
    GtkCListRow *row1;
    GtkCListRow *row2;
    gint        p1;
    gint        p2;
    glong       days1;
    glong       days2;
    glong       cdate;

    /*
     * FIXME: this function does not do what it should ...
     */
    
    row1 = (GtkCListRow *)ptr1;
    row2 = (GtkCListRow *)ptr2;

    cdate = date2days(current_date());
    days1 = get_note_days((note_data *)row1->data);
    days2 = get_note_days((note_data *)row2->data);

    p1 = get_note_prio(((note_data *)row1->data));
    p2 = get_note_prio(((note_data *)row2->data));

    p1 = cdate - days1 + p1;
    p2 = cdate - days1 + p2;

    return (p1 - p2);
}

/* ------------------------------------------------------ */
/*
 * return the number of days since 1.1.1970
 */

gint
get_note_days(note_data *note)
{
    static gchar fdate[] = "26 Feb 2038";
    gchar        *dptr;
    
    if (note && note->date)
    {
        dptr = note->date;
    }
    else
    {
        dptr = fdate;
    }
    if (!strlen(dptr))
    {
        dptr = fdate;
    }

    return (date2days(dptr));
}

/* ------------------------------------------------------ */
/*
 * return the note priority
 */

gint
get_note_prio(note_data *note)
{
    gint p;
    
    if (note)
    {
        if (strlen(note->prio))
        {    
            p = (gint) g_strtod(note->prio, NULL);
        }
        else
        {
            p = G_MAXINT / 2;
        }
    }
    else
    {
        p = G_MAXINT / 2;
    }

    return (p);
}

/* ------------------------------------------------------ */
